@@ -298,14 +298,14 @@ void libvisio::VSDXParser::handleStencils(WPXInputStream *input, unsigned shift)
     switch (ptrType)
     {
     case VSD_STENCIL_PAGE:
-      {
-        VSDXStencil tmpStencil;
-        m_currentStencil = &tmpStencil;
-        handleStencilPage(&tmpInput, shift2);
-        m_stencils.addStencil(i, *m_currentStencil);
-        m_currentStencil = 0;
-      }
-      break;
+    {
+      VSDXStencil tmpStencil;
+      m_currentStencil = &tmpStencil;
+      handleStencilPage(&tmpInput, shift2);
+      m_stencils.addStencil(i, *m_currentStencil);
+      m_currentStencil = 0;
+    }
+    break;
     default:
       break;
     }
@@ -499,6 +499,12 @@ void libvisio::VSDXParser::handleStencilShape(WPXInputStream *input)
       case VSD_FOREIGN_DATA:
         readForeignData(input);
         break;
+      case VSD_OLE_LIST:
+        readOLEList(input);
+        break;
+      case VSD_OLE_DATA:
+        readOLEData(input);
+        break;
       case VSD_FILL_AND_SHADOW:
         readFillAndShadow(input);
         break;
@@ -573,21 +579,21 @@ void libvisio::VSDXParser::_handleLevelChange(unsigned level)
   }
   if (level < 2)
   {
-    for (std::vector<VSDXGeometryList *>::iterator iter = m_geomListVector.begin(); iter != m_geomListVector.end(); iter++)
+    for (std::vector<VSDXGeometryList *>::iterator iter = m_geomListVector.begin(); iter != m_geomListVector.end(); ++iter)
     {
       (*iter)->handle(m_collector);
       (*iter)->clear();
       delete *iter;
     }
     m_geomListVector.clear();
-    for (std::vector<VSDXCharacterList *>::iterator iter2 = m_charListVector.begin(); iter2 != m_charListVector.end(); iter2++)
+    for (std::vector<VSDXCharacterList *>::iterator iter2 = m_charListVector.begin(); iter2 != m_charListVector.end(); ++iter2)
     {
       (*iter2)->handle(m_collector);
       (*iter2)->clear();
       delete *iter2;
     }
     m_charListVector.clear();
-    for (std::vector<VSDXParagraphList *>::iterator iter3 = m_paraListVector.begin(); iter3 != m_paraListVector.end(); iter3++)
+    for (std::vector<VSDXParagraphList *>::iterator iter3 = m_paraListVector.begin(); iter3 != m_paraListVector.end(); ++iter3)
     {
       (*iter3)->handle(m_collector);
       (*iter3)->clear();
@@ -683,6 +689,12 @@ void libvisio::VSDXParser::handlePage(WPXInputStream *input)
       case VSD_FOREIGN_DATA:
         readForeignData(input);
         break;
+      case VSD_OLE_LIST:
+        readOLEList(input);
+        break;
+      case VSD_OLE_DATA:
+        readOLEData(input);
+        break;
       case VSD_PAGE_PROPS:
         readPageProps(input);
         break;
@@ -780,6 +792,22 @@ void libvisio::VSDXParser::readForeignData(WPXInputStream *input)
   m_collector->collectForeignData(m_header.id, m_header.level, binaryData);
 }
 
+void libvisio::VSDXParser::readOLEList(WPXInputStream * /* input */)
+{
+  m_collector->collectOLEList(m_header.id, m_header.level);
+}
+
+void libvisio::VSDXParser::readOLEData(WPXInputStream *input)
+{
+  unsigned long tmpBytesRead = 0;
+  const unsigned char *buffer = input->read(m_header.dataLength, tmpBytesRead);
+  if (m_header.dataLength != tmpBytesRead)
+    return;
+  WPXBinaryData oleData(buffer, tmpBytesRead);
+
+  m_collector->collectOLEData(m_header.id, m_header.level, oleData);
+}
+
 void libvisio::VSDXParser::readEllipse(WPXInputStream *input)
 {
   input->seek(1, WPX_SEEK_CUR);
@@ -1040,7 +1068,14 @@ void libvisio::VSDXParser::readShapeList(WPXInputStream *input)
 
 void libvisio::VSDXParser::readForeignDataType(WPXInputStream *input)
 {
-  input->seek(0x24, WPX_SEEK_CUR);
+  input->seek(1, WPX_SEEK_CUR);
+  double imgOffsetX = readDouble(input);
+  input->seek(1, WPX_SEEK_CUR);
+  double imgOffsetY = readDouble(input);
+  input->seek(1, WPX_SEEK_CUR);
+  double imgWidth = readDouble(input);
+  input->seek(1, WPX_SEEK_CUR);
+  double imgHeight = readDouble(input);
   unsigned foreignType = readU16(input);
   input->seek(0xb, WPX_SEEK_CUR);
   unsigned foreignFormat = readU32(input);
@@ -1051,9 +1086,13 @@ void libvisio::VSDXParser::readForeignDataType(WPXInputStream *input)
     m_stencilShape.m_foreign->typeLevel = m_header.level;
     m_stencilShape.m_foreign->type = foreignType;
     m_stencilShape.m_foreign->format = foreignFormat;
+    m_stencilShape.m_foreign->offsetX = imgOffsetX;
+    m_stencilShape.m_foreign->offsetY = imgOffsetY;
+    m_stencilShape.m_foreign->width = imgWidth;
+    m_stencilShape.m_foreign->height = imgHeight;
   }
   else
-    m_collector->collectForeignDataType(m_header.id, m_header.level, foreignType, foreignFormat);
+    m_collector->collectForeignDataType(m_header.id, m_header.level, foreignType, foreignFormat, imgOffsetX, imgOffsetY, imgWidth, imgHeight);
 }
 
 void libvisio::VSDXParser::readPageProps(WPXInputStream *input)
@@ -1143,7 +1182,6 @@ void libvisio::VSDXParser::readNURBSTo(WPXInputStream *input)
   unsigned cellRef = 0;
   unsigned length = 0;
   unsigned long inputPos = input->tell();
-  unsigned long bytesRead = 0;
   while (cellRef != 6 && !input->atEOS() &&
          m_header.dataLength - chunkBytesRead > 4)
   {
@@ -1159,12 +1197,12 @@ void libvisio::VSDXParser::readNURBSTo(WPXInputStream *input)
   if (input->atEOS())
     return;
 
-  unsigned degree = 3;
-  unsigned char xType = 1;
-  unsigned char yType = 1;
   // Only read formula if block is found
   if (cellRef == 6)
   {
+    unsigned char xType = 1;
+    unsigned char yType = 1;
+    unsigned degree = 3;
     // Indicates whether it's a "simple" NURBS block with a static format
     // or a complex block where parameters each have a type
     unsigned char paramType = readU8(input);
@@ -1199,7 +1237,7 @@ void libvisio::VSDXParser::readNURBSTo(WPXInputStream *input)
     }
 
     // Read sequences of (x, y, knot, weight) until finished
-    bytesRead = input->tell() - inputPos;
+    unsigned long bytesRead = input->tell() - inputPos;
     unsigned char flag = 0;
     if (paramType != 0x8a) flag = readU8(input);
     while ((flag != 0x81 || (paramType == 0x8a && repetitions > 0)) && bytesRead < length)
@@ -1300,7 +1338,6 @@ void libvisio::VSDXParser::readPolylineTo(WPXInputStream *input)
   unsigned cellRef = 0;
   unsigned length = 0;
   unsigned long inputPos = input->tell();
-  unsigned long blockBytesRead = 0;
   while (cellRef != 2 && !input->atEOS() &&
          m_header.dataLength - chunkBytesRead > 4)
   {
@@ -1319,13 +1356,14 @@ void libvisio::VSDXParser::readPolylineTo(WPXInputStream *input)
     return;
 
   // Default to local co-ordinates if unspecified
-  unsigned char xType = 1;
-  unsigned char yType = 1;
   std::vector<std::pair<double, double> > points;
 
   // Only formula if block is found
   if (cellRef == 2)
   {
+    unsigned char xType = 1;
+    unsigned char yType = 1;
+    unsigned long blockBytesRead = 0;
     inputPos = input->tell();
     blockBytesRead += 6;
 
@@ -1501,39 +1539,6 @@ void libvisio::VSDXParser::readSplineKnot(WPXInputStream *input)
     m_geomList->addSplineKnot(m_header.id, m_header.level, x, y, knot);
 }
 
-void libvisio::VSDXParser::readParaIX(WPXInputStream *input)
-{
-  unsigned charCount = readU32(input);
-  input->seek(1, WPX_SEEK_CUR);
-  double indFirst = readDouble(input);
-  input->seek(1, WPX_SEEK_CUR);
-  double indLeft = readDouble(input);
-  input->seek(1, WPX_SEEK_CUR);
-  double indRight = readDouble(input);
-  input->seek(1, WPX_SEEK_CUR);
-  double spLine = readDouble(input);
-  input->seek(1, WPX_SEEK_CUR);
-  double spBefore = readDouble(input);
-  input->seek(1, WPX_SEEK_CUR);
-  double spAfter = readDouble(input);
-  unsigned char align = readU8(input);
-
-  if (m_isInStyles)
-    m_collector->collectParaIXStyle(m_header.id, m_header.level, charCount, indFirst, indLeft, indRight,
-                                    spLine, spBefore, spAfter, align);
-  else if (m_isStencilStarted)
-  {
-    VSD_DEBUG_MSG(("Found stencil paragraph style\n"));
-    if (!m_stencilShape.m_paraStyle)
-      m_stencilShape.m_paraStyle= new VSDXParaStyle(charCount, indFirst, indLeft, indRight,
-          spLine, spBefore, spAfter, align);
-  }
-  else
-    m_paraList->addParaIX(m_header.id, m_header.level, charCount, indFirst, indLeft, indRight,
-                          spLine, spBefore, spAfter, align);
-}
-
-
 void libvisio::VSDXParser::readNameList(WPXInputStream * /* input */)
 {
   if (m_isStencilStarted)
